xquery version "3.1";

(:~
 : This modules handles the conversion of the Fontante-TEI/XML into TEI simplePrint
 : for the edited text. The resulting TEI simplePrint is the basis for the "Editerter
 : Text" (edited text) view on the website and the book. It represents the latest
 : layer of text.
 :
 : @author Michelle Weidling
 : @version 0.1
 : @since TODO
 :)

module namespace fontaneSimple="http://fontane-nb.dariah.eu/teisimple";

declare namespace svg="http://www.w3.org/2000/svg";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace test="http://exist-db.org/xquery/xqsuite";
declare namespace xlink="http://www.w3.org/1999/xlink";

import module namespace config="http://textgrid.de/ns/SADE/config" at "../../config/config.xqm";
import module namespace functx="http://www.functx.com";
import module namespace simpleHelpers="http://fontane-nb.dariah.eu/teisimplehelpers" at "teisimplehelpers.xqm";
import module namespace tbleapi="https://fontane-nb.dariah.eu/tble" at "../tble-api.xqm";
import module namespace index-info="http://fontane-nb.dariah.eu/index-info" at "index-info.xqm";

(:~
 : The main function initiates the transformation of a given notebook.
 :
 : TODO: adapt to several input files?
 :
 : :)
declare function fontaneSimple:main($doc as node()*, $uri as xs:string, $log as
xs:string) as node()? {
(:declare function fontaneSimple:main($file as xs:string) as node()? {:)
(:  let $doc :=:)
(:    try {:)
(:      (doc("/db/sade-projects/textgrid/data/xml/data/" || $file)):)
(:    } catch * {:)
(:      (console:log("It was not possible to open the requested file " || $file)):)
(:    }:)

    let $front-covers := $doc//tei:sourceDoc/tei:surface[contains(@n, "front_cover")]
    let $back-covers := $doc//tei:sourceDoc/tei:surface[contains(@n, "back_cover")]
    let $content := ($doc//tei:sourceDoc/tei:surface[not(contains(@n, "cover")
        or matches(@n, "spine"))], $doc//tei:sourceDoc/tei:note[@type = "editorial"])

    let $tei :=
        <teiCorpus xmlns="http://www.tei-c.org/ns/1.0">
          <TEI id="{$doc//tei:teiHeader//tei:titleStmt/tei:title/string()}">
            {$doc//tei:teiHeader}
            <text>
                <front>{fontaneSimple:transform($front-covers, $uri, $log)}</front>
                <body>{fontaneSimple:transform($content, $uri, $log)}</body>
                <back>{fontaneSimple:transform($back-covers, $uri, $log)}</back>
            </text>
          </TEI>
        </teiCorpus>

    let $store := xmldb:store($config:data-root || "/print/xml/", $uri || "-tmp.xml", $tei)
    return $tei
};

(:~
 : Recursivly iterates the passed nodes and converts them according to the
 : requirements for the "Edierter Text". While it convers almost all of the
 : requirements stated in the encoding documentation (c.f.
 : https://fontane-nb.dariah.eu/doku.html), some parts of it are handled in a
 : second step (c.f. TODO) - especially the removal of tei:handShift duplicates
 : and the tei:milestone expansion to tei:div[@type = "section"] resp. tei:p -
 : because it is easier to perform these steps after the XML hierarchy has been
 : flattened a bit.
 :
 : @author Michelle Weidling
 : @param $nodes the elements of the book covers and the book content
 : @return $node()* a TEI simplePrint element
 :)
declare function fontaneSimple:transform($nodes as node()*, $uri as xs:string,
$log as xs:string) as node()* {
    for $node in $nodes
      return
        typeswitch ($node)
        case text() return
            if($node/ancestor::tei:line
            or $node/ancestor::tei:figDesc
            or $node/ancestor::tei:desc[@type = "edited_text"]
            or $node/ancestor::tei:note[@type = "editorial"]
            or $node/ancestor::tei:seg[@type = "editorial-label"]
            or $node/ancestor::tei:ref[not(matches(@target, "getty") or matches(@target, "xpath"))]) then
                if($node/parent::tei:rs and starts-with($node, " ")
                and not($node/preceding-sibling::*[1][self::tei:handShift or self::tei:hi])
                and simpleHelpers:is-trimming-necessary($node)) then
                    simpleHelpers:prepare-text(text{substring-after($node, " ")})
                else
                    simpleHelpers:prepare-text($node)
            else
                ()

        case element(tei:lb) return
            if($node[@break = "keepHyphen"]) then
                ()

            else if($node/ancestor::tei:seg[@type = "editorial-label"]) then
                text{" "}

            else if($node[@break = "no"]) then
                fontaneSimple:mark-intervention($node, $uri, $log)

            else
                fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:g) return
            if($node[@ref = "#vds"]) then
                ()
            else if($node/@ref ="#rth" or $node/@ref ="#hb") then
                fontaneSimple:copy-element($node, $uri, $log)
            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:del) return
            if($node/parent::tei:restore) then
                fontaneSimple:transform($node/node(), $uri, $log)
            else if($node/descendant::tei:restore) then
                fontaneSimple:transform($node/descendant::tei:restore, $uri, $log)
            else
                ()

        case element(tei:restore) return
            if(count($node/child::*) = 1 and $node/child::tei:del
            and $node/ancestor::tei:del) then
                ()
            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:retrace) return
            if($node/@rend) then
                element tei:seg {
                    attribute rendition {$node/@rend},
                    fontaneSimple:transform($node/node(), $uri, $log)
                }
            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:add) return
            if($node/@type = "edited_text"
            or $node/child::tei:seg[@type = "multiphrase"]) then
                fontaneSimple:copy-element($node, $uri, $log)

            else if($node/@cause = "unclear") then
                ()

            else if($node[replace($node/@copyOf, "#", "") = $node/ancestor::tei:TEI//tei:seg/@xml:id]) then
                fontaneSimple:mark-intervention($node, $uri, $log)

            else if($node/@cause ="catchword") then
                ()

            else if($node/@rend ="|") then
                fontaneSimple:transform($node/node(), $uri, $log)

            else if($node/@place = "above"
            and $node/preceding::node()[1][normalize-space(.) = ""]) then
                (text{" "},
                fontaneSimple:transform($node/node(), $uri, $log))

            else if($node[@place = "superimposed"]) then
                fontaneSimple:preserve-whitespace($node, $uri, $log)


            else if(not($node/@xml:id)) then
                fontaneSimple:transform($node/node(), $uri, $log)

            else if(simpleHelpers:is-transposed($node)) then
                let $corresp := //tei:metamark[matches(@target, $node/@xml:id)]
                return
                    (fontaneSimple:transform($corresp/node(), $uri, $log),
                    fontaneSimple:transform($node/node(), $uri, $log))

            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:addSpan) return
            if($node/@type = "edited_text") then
                fontaneSimple:copy-element($node, $uri, $log)
            else
                ()

        case element(tei:fw) return
            ()

        case element(tei:line) return
            if($node[ancestor::tei:seg[@type = "editorial-label"]]) then
                fontaneSimple:transform($node/node(), $uri, $log)

            else if($node/@type = "heading") then
                fontaneSimple:make-head($node, $uri, $log)

            else if(simpleHelpers:has-valid-style($node)
            or matches($node/@rendition, "black_letter")
            or matches($node/@rendition, "roman")) then
                fontaneSimple:make-seg-with-rendition($node, $uri, $log)

            else if($node/following::*[1][self::tei:lb[@break = "no"]]) then
                (simpleHelpers:start-line($node),
                simpleHelpers:trim-last-char($node))

            else if ($node/preceding::*[1][self::tei:lb[@break = "no"]]) then
                (simpleHelpers:start-line($node),
                simpleHelpers:trim-first-char($node))

            else if($node/@type = "verse") then
                if($node/@prev) then
                    ()

                else if(not($node/@next)) then
                    (if(not($node/preceding-sibling::tei:line[@type = "verse"])) then
                        fontaneSimple:mark-linegroup-beginning()
                    else
                        (),
                    element tei:l {
                            fontaneSimple:transform($node/node(), $uri, $log)
                        },
                    if(not($node/following-sibling::tei:line[@type = "verse"])) then
                        fontaneSimple:mark-linegroup-end()
                    else
                        ())

                (: 3.8.2.2.1.3 Vers mit anderer Beschriftung in einer Zeile :)
                else
                    let $corresp := $node/following::*[@type = "verse"
                        and replace($node/@next, "#", "") = @xml:id]
                    return
                        (fontaneSimple:mark-linegroup-beginning(),
                        element tei:l {
                            fontaneSimple:transform($node/node(), $uri, $log),
                            simpleHelpers:start-line($node),
                            fontaneSimple:transform($corresp/node(), $uri, $log)
                        },
                        fontaneSimple:mark-linegroup-end())

            else if($node/parent::tei:zone[@type = "verse"]/child::*[1] = $node) then
                fontaneSimple:transform($node/node(), $uri, $log)

            (: with opting for a tei:milestone we try to meet the project's
            requirements on the one hand while avoiding to run into hierarchical
            problems in the further processing on the other hand:)
            else if(not($node/@type = "item")) then
                (simpleHelpers:start-line($node),
                fontaneSimple:transform($node/node(), $uri, $log))

            else if($node/@type = "item") then
                element tei:item {
                    $node/(@* except (@rend, @type, @style)),
                    if($node/@rend) then
                        attribute rendition {$node/@rend}
                    else
                        (),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else
                (simpleHelpers:start-line($node),
                fontaneSimple:transform($node/node(), $uri, $log))

        case element(tei:handShift) return
            if($node/ancestor::tei:seg[@type = "editorial-label"]) then
                ()
            else
                fontaneSimple:enhance-handshift($node)

        case element(tei:stamp) return
            if($node/ancestor::tei:surface[matches(@n, "Beilage")]) then
                fontaneSimple:copy-element($node, $uri, $log)
            else
                ()

        case element(tei:seg) return
            if(count($node/*) = 1 and
            (($node/tei:stamp
                and not($node/ancestor::tei:surface[matches(@n, "Beilage")]))
            or $node/tei:metamark[@function = "caret"]))
                then
                    ()

            else if($node/@type = "heading") then
                fontaneSimple:make-head($node, $uri, $log)

            else if(matches($node/@style, "underline")
            and not(matches($node/@style, "vertical-align"))) then
                fontaneSimple:transform($node/node(), $uri, $log)

            else if(simpleHelpers:has-valid-style($node)
            or matches($node/@rendition, "black_letter")
            or matches($node/@rendition, "roman")) then
                fontaneSimple:make-seg-with-rendition($node, $uri, $log)

            else if($node/@type = "initials"
            or $node/@type = "monogram"
            or $node/@type = "multiphrase"
            or $node/@xml:lang)
                then
                    (fontaneSimple:copy-element($node, $uri, $log),
                    if($node/@type = "multiphrase") then
                        text{" "}
                    else
                        ())

            else if($node/@type = "auction_number"
            or $node/@type = "cancel"
            or $node/@type = "abort"
            or $node/@function ="unknown")
                then
                    ()

            else if($node/parent::tei:add and $node/@copyOf) then
                fontaneSimple:mark-intervention($node, $uri, $log)

            else if($node/@type = "verse" and $node/@prev) then
                ()

            else if($node/@type = "said") then
                if($node/@next) then
                    let $next := replace($node/@next, "#", "")
                    let $corresp := $node/following::*[@xml:id = $next]
                    return
                        (element tei:seg {
                            $node/@type,
                            fontaneSimple:transform($node/node(), $uri, $log),
                            fontaneSimple:transform($corresp/node(), $uri, $log)
                        },
                        element tei:lb {
                            attribute type {"edited_text"}
                        })
                else
                    element tei:seg {
                        $node/@type,
                        fontaneSimple:transform($node/node(), $uri, $log)
                    }

            else if($node/@type = "item") then
                element {QName("http://www.tei-c.org/ns/1.0", $node/@type)} {
                    $node/(@xml:id, @subtype, @rendition, @prev, @next),
                    if($node/@rend) then
                        attribute rendition {$node/@rend}
                    else
                        (),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = ("integration", "editorial-label")) then
                element tei:seg {
                    $node/@*,
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:hi) return
            (element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                $node/@*,
                fontaneSimple:transform($node/node(), $uri, $log)
            },
            if(not($node/@prev or $node/@next)
            and $node/following::node()[1][self::text()][normalize-space(.) = ""]
            and $node/following::*[1][self::tei:hi]) then
                text{"@"}
            else
                ()
                )

        (: TODO if $node/@type = "highlighted" then make
        a hi[@type = "vertical-mark"] in the second stage of creating the
        simple format. use simpleHelpers:get-xml-chunk($node) for this.:)
        case element(tei:mod) return
            if($node/@type = "highlighted"
            and simpleHelpers:is-hand-contemporary($node/@hand)) then
                $node
            else if($node/following::node()[1][self::text()]) then
                fontaneSimple:preserve-whitespace($node, $uri, $log)
            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:anchor) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:surface) return
            if(matches($node/@n, "cover")) then
                (fontaneSimple:make-pb-with-type($node/@n),
                fontaneSimple:transform($node/node(), $uri, $log))

            else if(matches($node/@n, "Beilage")) then
                element tei:div {
                    $node/@*,
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "pocket") then
                element tei:div{
                    $node/(@* except (@n, @ulx, @uly, @lry, @lrx)),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if(simpleHelpers:is-page($node)
            and $node/@type = "clipping") then
                if($node/@subtype = "Kalenderblatt"
                and contains($node//tei:handShift/@new, "Friedrich_Fontane")) then
                    element tei:div {
                        $node/@n,
                        attribute type {$node/@subtype},
                        fontaneSimple:transform($node/node(), $uri, $log)
                    }

                else if(not($node/@subtype = "Kalenderblatt"
                or $node/@subtype = "Zeitungsausschnitt_Fragment")) then
                    element tei:div {
                        $node/(@* except (@n, @ulx, @uly, @lry, @lrx)),
                        fontaneSimple:transform($node/node(), $uri, $log)
                    }

                else
                    ()

            else if(simpleHelpers:is-page($node)) then
                (text{" "},
                fontaneSimple:transform($node/node(), $uri, $log))

            else if($node/@type = "label" and
            (contains($node/@subtype, "Fontane")
            or contains($node/@subtype, "Hersteller"))
            ) then
                fontaneSimple:make-div($node, $uri, $log)

            else
                ()

        case element(tei:milestone) return
            if($node/@unit = "illustration") then
                ()
            else
                fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:gap) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:metamark) return
            if($node/@function = "integrate"
            or $node/@function = "authorial_note") then
                element tei:ab {
                    $node/@*
                }
            else if($node/@function = ("placeholder", "etc.", "caret",
            "footnote-mark", "footnotes", "ellipsis", "paragraph")) then
                element tei:ab {
                    attribute type {$node/@function},
                    fontaneSimple:transform($node/node(), $uri, $log)
                }
            else
                ()

        case element(tei:surplus) return
            fontaneSimple:mark-intervention($node, $uri, $log)

        case element(tei:zone) return
            if(matches($node/@style, "border-style:solid")
            and not(matches($node/@style, "border-radius"))
            and not($node/@rend = "border-style:house")) then
                element tei:div {
                    attribute type {"frame"},
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if(matches($node/@rend, "border-bottom-style:brace")) then
                (fontaneSimple:transform($node/node(), $uri, $log),
                element tei:ab {
                    attribute type {"bottom-brace"}
                })

            else if($node/@type = "cancel") then
                ()

            else if($node/@type = "marked_off") then
                element tei:seg {
                    $node/@type,
                    $node/@xml:id,
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "highlighted") then
                if($node/child::tei:zone[@type = "highlighted"]) then
                    fontaneSimple:transform($node/node(), $uri, $log)
                else
                    element tei:hi {
                        $node/@xml:id,
                        $node/@prev,
                        $node/@next,
                        attribute type {"vertical-mark"},
                        fontaneSimple:transform($node/node(), $uri, $log)
                    }

            else if($node/@type = "illustration"
            or $node/@type = "printed_illustration") then
                if(not($node//tei:figure/parent::tei:del)) then
                    element tei:ab {
                        (if($node/child::tei:zone[@type = "illustration"]) then
                            attribute type {"composed-sketch"}
                        else
                            (attribute type {"sketch"},
                            if($node/parent::tei:zone[@type = "illustration"]) then
                                attribute rendition {"margin-left:" || $node/@ulx
                                || "cm; " || "margin-top:" || $node/@uly || "cm"}
                            else
                                ()
                            )
                        ),
                        attribute width {$node/@lrx - $node/@ulx || "cm"},
                        attribute height {$node/@lry - $node/@uly || "cm"},
                        fontaneSimple:transform($node/node(), $uri, $log)
                    }
                else
                    ()

            else if($node/parent::tei:zone/@type = "illustration"
            or $node/parent::tei:zone/@type = "printed_illustration") then
                element tei:seg {
                    attribute type {"caption"},
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "heading") then
                fontaneSimple:make-head($node, $uri, $log)

            else if($node/@type = "list"
            or $node/@type = "item") then
                element {QName("http://www.tei-c.org/ns/1.0", $node/@type)} {
                    $node/(@xml:id, @subtype, @rendition, @prev, @next),
                    if($node/@rend) then
                        attribute rendition {$node/@rend}
                    else
                        (),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "dialogue") then
                element tei:div {
                    $node/@type,
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "verse") then
                (fontaneSimple:mark-linegroup-beginning(),
                element tei:l {
                    fontaneSimple:transform($node/node(), $uri, $log)
                },
                fontaneSimple:mark-linegroup-end())

            else if($node/@type = "said" and $node/@prev) then
                ()

            else if($node/@type = "said") then
                element tei:seg {
                    $node/@type,
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "toc"
            and matches($node/@subtype, "ungültig")) then
                ()

            else if($node/@type = "toc") then
                element tei:div {
                    $node/(@type, @subtype),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "legend") then
                element tei:div {
                    (if($node/@style
                    or $node/@rendition) then
                        attribute rendition {simpleHelpers:filter-rendition($node)}
                    else
                        ()),
                    $node/(@* except (@rendition, @style)),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if($node/@type = "additional") then
                element tei:ab {
                    $node/(@* except (@ulx, @uly, @lry, @lrx)),
                    fontaneSimple:transform($node/node(), $uri, $log)
                }

            else if(simpleHelpers:has-valid-style($node)
            or matches($node/@rendition, "black_letter")
            or matches($node/@rendition, "script")
            or matches($node/@rendition, "roman")) then
                fontaneSimple:make-seg-with-rendition($node, $uri, $log)

            else if(not($node/@xml:id)) then
                fontaneSimple:transform($node/node(), $uri, $log)

            else if($node/@xml:id) then
                (
(:                    if($node/@xml:id = "C06_14r_9") then:)
(:                    util:log-system-out($node):)
(:                else (),:)
                element tei:seg {
                    $node/@xml:id,
                    $node/@prev,
                    $node/@next,
                    $node/@corresp,
                    fontaneSimple:transform($node/node(), $uri, $log)
                })

            else
                fontaneSimple:transform($node/node(), $uri, $log)

        case element(tei:said) return
            (element tei:seg {
                attribute type {"said"},
                fontaneSimple:transform($node/node(), $uri, $log)
            },
            if($node/ancestor::tei:zone[@type = "dialogue"]/descendant::tei:said[position() != last()] = $node) then
                element tei:lb {
                        attribute type {"edited_text"}
                }
            else
                ())

        case element(tei:figure) return
(:            if(count($node/child::*) = 2 :)
(:            and $node/child::tei:figDesc:)
(:            and $node/child::tei:ptr) then:)
                (: genealogy lines probably shouldn't be displayed, but I still have to
                check that. in case they should be serialized, I leave the code :)
(:                if(matches($node/descendant::tei:ref, "Stammbaumverbindungslinie")) then:)
(:                    element tei:seg {:)
(:                        $node/@*,:)
(:                        fontaneSimple:transform($node/node(), $uri, $log):)
(:                    }:)
(:                else if(matches($node/descendant::tei:ref, "Schlusslinie")):)
                if(matches($node/descendant::tei:ref, "Schlusslinie"))
                    then
                        element tei:ab {
                            switch ($node/descendant::tei:ref)
                                case "horizontale einfache Schlusslinie" return
                                    attribute type {"long-end-line"}
                                case "Schlusslinie; horizontale Halbschleife von links oben nach rechts" return
                                    attribute type {"long-end-line"}
                                case "horizontale einfache Schlusslinie (gewellt)" return
                                    attribute type {"long-end-line-wavy"}
                                case "Schlusslinien; horizontale Schleife von links oben nach rechts unten" return
                                    attribute type {"bottom-brace-short"}
                                default return
                                    attribute type {"end-line"}
                        }

                else if(matches($node/descendant::tei:ref, "Absatzlinie")
                (: in case of double paragraph lines the single lines are
                encoded with "oberer" resp. "unterer Teil", but we only
                serialize the encoding for the upper line :)
                and not(matches($node/descendant::tei:ref, "unterer Teil"))
                and not(matches($node/descendant::tei:figDesc, "unsicher"))) then
                    element tei:ab {
                        if(matches($node/descendant::tei:ref, "doppelt"))
                            then
                                attribute type {"short-paragraph-line-double"}
                            else
                                attribute type {"short-paragraph-line"}
                    }

                else if(matches($node/descendant::tei:ref, "horizontale einfache Abgrenzungslinie")) then
                    ()

                else if($node/ancestor::tei:zone[@type = "illustration"]) then
                    let $display :=
                        if($node/ancestor::tei:zone[@type = "illustration"]/tei:milestone[@unit = "illustration"][. << $node]) then
                            "block"
                        else
                            "inline"
                    let $img-url := try {tbleapi:get-url($uri, $node/@xml:id, "png") }
                        catch * { fontaneSimple:add-log-entry($log, "No TBLE-file found for " || serialize($node)) }
                    let $img-url :=
                        if($display = "inline") then
                            replace($img-url, ",1000", ",500")
                        else
                            $img-url
                    let $rotation :=
                        (: no children and comment right after element:)
                        if($node/following-sibling::node()[1][self::comment()][matches(./string(), "rotate")]
                        and not($node/child::*)) then
                            substring-after($node/following::comment()[1], "rotate(")
                            => substring-before("deg")
                        (: comment as first descendant :)
                        else if($node/child::node()[1][self::comment()][matches(./string(), "rotate")]) then
                            substring-after($node/child::comment()[1], "rotate(")
                            => substring-before("deg")
                        else
                            ()

                    return
                        element {QName("http://www.tei-c.org/ns/1.0", $node/name())}{
                            $node/@*,
                            attribute position {$display},
                            attribute href {$img-url},
                            if($rotation) then
                                attribute rotate {$rotation}
                            else
                                (),
                            fontaneSimple:transform($node/node(), $uri, $log)
                        }
                else

                    ()
(:            else:)
(:                fontaneSimple:copy-element($node, $uri, $log):)

        case element(tei:note) return
            if($node/@type = "authorial"
            and not($node/@subtype = "footnote")) then
                ()
            else
                fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:certainty) return
            element tei:note {
                attribute type {"editorial"},
                attribute subtype {"certainty"},
                $node/@cert,
                $node/@target,
                fontaneSimple:transform($node/node(), $uri, $log)
            }

        case element(tei:figDesc) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:ref) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:space) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:choice) return
            if($node[ancestor::tei:seg[@type = "editorial-label"]]) then
                $node/tei:abbr
            else
                fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:abbr) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:expan) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:sic) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:corr) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:rs) return
            let $index-type := substring-before($node/@ref, ":")
            return
                (: ignore tei:rs elements in labels for the editorial commentary :)
                if($node[ancestor::tei:seg[@type = "editorial-label"]]) then
                    fontaneSimple:transform($node/node(), $uri, $log)

                (: ignore tei:rs elements that are empty after presorting. this
                is important because they would cause a lot of whitespace trouble
                otherwise :)
                else if($node//text()[matches(., "[\w]")]) then
                    (
                        element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                        $node/@*,
                        if($node/@prev) then
                            ()
                        else
                            fontaneSimple:make-index-infos($node, $index-type),
                        fontaneSimple:transform($node/node(), $uri, $log)
                    },
                    if(not($node//tei:abbr/text()[ends-with(., ":")]
                    or ($node//text()[last()][ends-with(., "-")] or $node/following-sibling::*[1][self::tei:rs])
                    or $node/following::text()[1][starts-with(., "-") or starts-with(., ".") or starts-with(., ")")]
                    or $node/following::text()[1][matches(substring(., 1, 1), "[\w]")]
                    or $node/following::*[1][self::tei:handShift])
(:                    or $node/following-sibling::node()[1][starts-with(., "-") or starts-with(., ".")]):)
                    ) then
                        text{" "}
                    else
                        ()
                    )
                else
                    ()

        case element(tei:date) return
            element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                (if($node/@when-iso) then
                    attribute when {$node/@when-iso}
                else
                    ()),
                (if($node/@from-iso) then
                    attribute from {$node/@from-iso}
                else
                    ()),
                (if($node/@to-iso) then
                    attribute to {$node/@to-iso}
                else
                    ()),
                (if($node/@notAfter-iso) then
                    attribute notAfter {$node/@notAfter-iso}
                else
                    ()),
                $node/(@* except (@when-iso, @to-iso, @from-iso, @notAfter-iso)),
                fontaneSimple:transform($node/node(), $uri, $log)
            }

        case element(tei:ptr) return
            if($node[@type = 'editorial-commentary']) then
                $node
            else if($node[ancestor::tei:bibl]) then
                if(fontaneSimple:has-only-ptr-as-child($node/ancestor::tei:bibl)) then
                    fontaneSimple:get-sec-lit-reference($node)
                else
                    fontaneSimple:copy-element($node, $uri, $log)
            else
                ()

        case element (tei:bibl) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:unclear) return
            fontaneSimple:copy-element($node, $uri, $log)

        case element(tei:div) return
            fontaneSimple:copy-element($node, $uri, $log)

        default return
            fontaneSimple:transform($node/node(), $uri, $log)
};

(:~
 : Takes a given element over as is since it is compliant to TEI simplePrint.
 :
 : @author Michelle Weidling
 : @param $node the current text node
 : @return node() a copy of the current node
 :)
declare function fontaneSimple:copy-element($node as node(), $uri as xs:string,
$log as xs:string) as node() {
    element {QName("http://www.tei-c.org/ns/1.0", $node/name())}{
        $node/(@* except @rend),
        if($node/@rend) then
            attribute rendition {$node/@rend}
        else
            (),
        fontaneSimple:transform($node/node(), $uri, $log)
    }
};

(:~
 : Creates a tei:head with an @rendition and a font-size, if available.
 :
 : @author Michelle Weidling
 : @param $node the current tei:line, tei:zone or tei:seg node
 : @return element(tei:head)
 : :)
declare function fontaneSimple:make-head($node as node(), $uri as xs:string,
$log as xs:string) as element(tei:head) {
    element tei:head {
        (if($node/descendant::tei:seg[matches(@style, "font-size")]) then
            attribute style {simpleHelpers:get-font-size($node)}
        else
            attribute style {"default"}
        ),

        $node/@subtype,

        (if($node//tei:hi) then
            attribute underlined {"true"}
        else
            ()),

        (if($node/@rendition) then
            $node/@rendition
        else
            ()),

        (if($node/@rend) then
            $node/@rend
        else
            ()),
        fontaneSimple:transform($node/node(), $uri, $log)
    }
};

(:~
 : Creates a tei:div.
 :
 : @author Michelle Weidling
 : @param $node the current tei:surface node
 : @return element(tei:div)
 : :)
declare function fontaneSimple:make-div($node as element(tei:surface), $uri as
xs:string, $log as xs:string) as element(tei:div) {
    element tei:div {
        $node/(@* except (@facs, @n, @attachment, @subtype, @ulx, @uly, @lrx, @lry, @points)),
        fontaneSimple:transform($node/node(), $uri, $log)
    }
};

(:~
 : Creates a new tei:seg with all relevant rendition information.
 :
 : @author Michelle Weidling
 : @param $node the current tei:zone, tei:line, or tei:seg element
 : @return element(tei:seg)
 :)
declare function fontaneSimple:make-seg-with-rendition($node as element(*),
$uri as xs:string, $log as xs:string) as element(tei:seg) {
    element tei:seg {
        attribute rendition {simpleHelpers:filter-rendition($node)},
        $node/(@* except (@style, @rendition)),
        (if($node[self::tei:line or self::tei:zone]) then
            attribute unit {$node/name()}
        else
            ()),
        fontaneSimple:transform($node/node(), $uri, $log)
    }
};


(:~
 : Creates a tei:pb.
 :
 : @author Michelle Weidling
 : @param $node the current tei:surface node
 : @return element(tei:pb)
 :)
declare function fontaneSimple:make-pb($node as element(tei:surface))
as element(tei:pb) {
    element tei:pb {
        $node/@n
    }
};


(:~
 : Creates a tei:pb with a type instead of a @n representing the pagination.
 : This function is mainly used to separate the inner and outer book covers from
 : each other.
 :
 : @author Michelle Weidling
 : @param $node the current tei:surface node
 : @return element(tei:pb)
 :)
declare function fontaneSimple:make-pb-with-type($type as xs:string)
as element(tei:pb) {
    element tei:pb {
        attribute type {$type}
    }
};

declare function fontaneSimple:enhance-handshift($node as element(tei:handShift))
as element(tei:milestone) {
    let $prev-hand := $node/preceding::tei:handShift[@new][1]
    let $whitespace-before :=
        if($node/preceding::node()[1][normalize-space(.) = ""]
        and string-length($node/preceding::text()[1]) = 1) then
            true()
        else
            false()

    let $whitespace-after :=
        if($node/following::node()[1][normalize-space(.) = ""]
        and string-length($node/following::text()[1]) = 1
        and not($node/following::*[1][@type = "heading"])
        and not($node/following::text()[1][matches(substring(., 1, 1), "[\.,]")])) then
            true()
        else
            false()

    return
        element tei:milestone {
            attribute unit {"handshift"},
            attribute subtype {if($node/@new) then $node/@new else $prev-hand/@new},
(:            (if($whitespace-before) then attribute ws-before {"true"} else ()),:)
(:            (if($whitespace-after) then attribute ws-after {"true"} else ()),:)

            $node/(@* except @new)
        }
};


declare function fontaneSimple:mark-intervention($node as element(*), $uri as
xs:string, $log as xs:string)
as element(tei:seg) {
    element tei:seg {
        if($node[self::tei:lb]) then
            attribute type {"missing-hyphen"}
        else if($node[self::tei:surplus]) then
            attribute type {"surplus"}
        else
            attribute type {"reduplication"},

        <tei:hi xmlns="http://www.tei-c.org/ns/1.0">‹</tei:hi>,

        if($node[self::tei:lb]) then
            simpleHelpers:find-chars($node)
        else
            fontaneSimple:transform($node/node(), $uri, $log),

        <tei:hi xmlns="http://www.tei-c.org/ns/1.0">›</tei:hi>
    }
};


declare function fontaneSimple:mark-missing-hyphen($node as element(tei:lb)) as
element(tei:seg) {
    element tei:seg {
        attribute type {"missing-hyphen"},
        <tei:hi xmlns="http://www.tei-c.org/ns/1.0">‹</tei:hi>,
        simpleHelpers:find-chars($node),
        <tei:hi xmlns="http://www.tei-c.org/ns/1.0">›</tei:hi>
    }
};


(:~ In this first serialization step the beginning and end of line groups are
 : simply marked with milestones, which are expanded to a full tei:lg in the
 : second serialization step. :)
declare function fontaneSimple:mark-linegroup-beginning() as
element(tei:milestone) {
    element tei:milestone {
        attribute unit {"start-lg"}
    }
};

declare function fontaneSimple:mark-linegroup-end() as
element(tei:milestone) {
    element tei:milestone {
        attribute unit {"end-lg"}
    }
};

declare function fontaneSimple:has-pb-inbetween($node as node(),
$corresp as node()*) as xs:boolean {
    if($node/following::tei:surface[. << $corresp]) then
        true()
    else
        false()
};

declare function fontaneSimple:separates-corresp-nodes($node as element(tei:surface))
as xs:boolean {
    some $prev in $node/preceding::*[@next]
    satisfies contains($prev/@next, $node/descendant::*/@xml:id)
        or contains($prev/@next, $node/following::*/@xml:id)
};

declare function fontaneSimple:make-index-infos($node as element(tei:rs),
$index-type as xs:string) as element()* {
    let $refs := tokenize($node/@ref, " ")
    let $no-of-refs := count($refs)

    return
        for $iii in 1 to $no-of-refs
        let $ref := $refs[$iii]
        return
            element tei:index {
                attribute indexName {$index-type},

                try {
                    element tei:term {
                        attribute type {"main"},
                        index-info:get-info-about($index-type, $ref, "regular-name")
                    },
                    element tei:term {
                        attribute type {"key"},
                        fontaneSimple:make-key($index-type, $ref, "regular-name")
                    },
                    switch ($index-type)
                        case "eve" return
                            let $category := index-info:get-info-about($index-type, $ref, "category")
                            return
                                fontaneSimple:make-term("subref-of", $category)
                        case "org" return
                            ()

                        case "plc" return
                            let $subref := index-info:get-info-about($index-type, $ref, "subref-of")
                            return
                                fontaneSimple:make-term("subref-of", $subref)

                        case "psn" return
                            let $birth := index-info:get-info-about($index-type, $ref, "birth")
                            let $death := index-info:get-info-about($index-type, $ref, "death")
                            let $occupation := index-info:get-info-about($index-type, $ref, "occupation")
                            return
                                (fontaneSimple:make-term("birth", $birth),
                                fontaneSimple:make-term("death", $death),
                                fontaneSimple:make-term("occupation", $occupation),
                                if(index-info:get-info-about($index-type, $ref, "relation-to-fontane")) then
                                    let $relation := index-info:get-info-about($index-type, $ref, "relation-to-fontane")
                                    return
                                        fontaneSimple:make-term("relation-to-fontane", $relation)
                                else
                                    (),
                                if(index-info:get-info-about($index-type, $ref, "same-as")) then
                                    let $same-as := index-info:get-info-about($index-type, $ref, "same-as")
                                    return
                                        fontaneSimple:make-term("same-as", $same-as)
                                else
                                    ())

                        case "wrk" return
                            if(index-info:get-info-about($index-type, $ref, "type") = "work") then
                                fontaneSimple:make-infos-about-work($ref, $index-type)

                            (: the index "wrk" not only encompasses works but
                            also fictional characters that occur in them. @part-of-id
                            denotes this relationship. to put the character into the
                            right place we also have to provide the whole informatiom
                            about the work. :)
                            else if(index-info:get-info-about($index-type, $ref, "type") = "person") then
                                let $work-id :=
                                    try {
                                        index-info:get-info-about($index-type, $ref, "part-of-id")
                                    } catch * {
                                        error(QName("error", "TEI2SIMPLE001"), "No work found for " || $ref || ".")
                                    }
                                let $work-name := index-info:get-info-about($index-type, $work-id, "regular-name")

                                return
                                    (
                                        fontaneSimple:make-infos-about-work($work-id, $index-type),
                                        fontaneSimple:make-infos-about-fictional-character($ref, $index-type, $work-name)
                                    )
                            else
                                ()

                        default return
                            ()
                }
                catch * {
                        fontaneSimple:make-term("key", $ref),
                        fontaneSimple:make-term("main", $ref),
                        element tei:term {
                            attribute type {"subref"},
                            fontaneSimple:make-term("key", "!!FEHLERHAFTERDATENSATZ"),
                            fontaneSimple:make-term("main", "!!FEHLERHAFTER DATENSATZ")
                        }
                }
            }
};

declare function fontaneSimple:make-term($type as xs:string, $info as xs:string)
as element(tei:term)* {
    if($info != "") then
        element tei:term {
            attribute type {$type},
            $info
        }
    else
        ()
};

declare function fontaneSimple:make-key($index-type as xs:string, $ref as xs:string, $term as xs:string) {
    let $main := index-info:get-info-about($index-type, $ref, $term)
    return
        fontaneSimple:make-key($main)
};

declare function fontaneSimple:make-key($regular-name as xs:string?) {
    let $name :=
        if(contains($regular-name, ".")) then
            substring-before($regular-name, ".")
        else
            $regular-name
    let $affix := substring-after($regular-name, ".")
    let $key :=
        replace($name, "Ä", "A")
        => replace("ä", "ae")
        => replace("Ö", "Oe")
        => replace("ö", "oe")
        => replace("Ü", "Ue")
        => replace("ü", "ue")
        => replace(" ", "")
        => replace("\.", "")
        => replace(",", "")
    return
        (: sorts terms with a trailer, e.g. 'Friederich II. von Sachsen' after the ones
            without, e.g. 'Friederich II.' :)
        if(matches($affix, "[\w]")) then
            $key || "A"
        else
            $key
};

(:~
 : Creates all information needed about a tei:rs[matches(@ref, 'wrk')] which includes:
 :  * the creator of the work
 :  * information about the work is a periodical or not
 :
 : @param $ref the string of tei:rs/@ref
 : @param $index-type "wrk"
 :   :)
declare function fontaneSimple:make-infos-about-work($ref as xs:string, $index-type
as xs:string) as element(tei:term)+ {
    let $subref-ids := index-info:get-info-about($index-type, $ref, "creator-ids")
    let $creator-ids :=tokenize($subref-ids, " ")
    (: Fontane or anonymus :)
    let $creator := index-info:get-info-about($index-type, $ref, "creator")
    let $creators :=
        if($creator) then
            $creator
        else
            for $creator-id in $creator-ids
            return
                if(matches($creator-id, "#")) then
                    ()
                else
                    $creator-id

    let $type := index-info:get-info-about($index-type, $ref, "periodical")
    let $same-as := index-info:get-info-about($index-type, $ref, "same-as")

    let $no-of-ids := count($creators)
    return
        (
        fontaneSimple:make-term("periodical", $type),

        if($same-as) then
            fontaneSimple:make-term("same-as", $same-as)
        else
            (),

        if($subref-ids) then
            for $iii in 1 to $no-of-ids return
                let $subref := index-info:get-info-about("psn", $creators[$iii], "regular-name")
                let $subref-key := fontaneSimple:make-key($subref)
                let $birth := index-info:get-info-about("psn", $creators[$iii], "birth")
                let $death := index-info:get-info-about("psn", $creators[$iii], "death")
                let $occupation := index-info:get-info-about("psn", $creators[$iii], "occupation")
                return
                    element tei:term {
                        attribute type {"subref"},
                        fontaneSimple:make-term("birth", $birth),
                        fontaneSimple:make-term("death", $death),
                        fontaneSimple:make-term("occupation", $occupation),
                        fontaneSimple:make-term("key", $subref-key),
                        fontaneSimple:make-term("main", $subref),
                        if(index-info:get-info-about("psn", $creators[$iii], "relation-to-fontane")) then
                            let $relation := index-info:get-info-about("psn", $creators[$iii], "relation-to-fontane")
                            return
                                fontaneSimple:make-term("relation-to-fontane", $relation)

                         else
                            ()
                    }
        else if(matches($creator, "Fontane")) then
            fontaneSimple:make-term("creator", $creator)

        else if(matches($creator, "Anonym")) then
            element tei:term {
                attribute type {"subref"},
                fontaneSimple:make-term("main", $creator)
            }

        else
            ())
};

declare function fontaneSimple:make-infos-about-fictional-character($ref as
xs:string, $index-type as xs:string, $work-name as xs:string) as element(tei:term) {
    let $regular-name := index-info:get-info-about($index-type, $ref, "regular-name")
    let $note := index-info:get-info-about($index-type, $ref, "note")

    return
        element tei:term {
            attribute type {"person-mentioned"},
            fontaneSimple:make-term("part-of-work", $work-name),
            fontaneSimple:make-term("regular-name", $regular-name),
            if($note) then
                fontaneSimple:make-term("note", $note)
             else
                ()
        }
};

declare function fontaneSimple:preserve-whitespace($node as element(*), $uri as
xs:string, $log as xs:string) as node()* {
    let $first-char := $node/following::node()[1][self::text()]
        => substring(1, 1)

    return
        (fontaneSimple:transform($node/node(), $uri, $log),
        if(matches($first-char, "[\s]") and matches($node/following::node()[1][self::text()], "[\w]")) then
             text{" "}
        else
            ())
};

declare function fontaneSimple:has-only-ptr-as-child($bibl as element(tei:bibl))
as xs:boolean {
    let $text-nodes := $bibl//text()
    let $have-nodes-content :=
        for $text in $text-nodes return
            if(normalize-space($text) != "") then
                true()
            else
                false()

    return
        if($have-nodes-content = true()) then
            false()
        else
            true()
};

declare function fontaneSimple:get-sec-lit-reference($ptr as element(tei:ptr))
as text() {
    let $reference := $ptr/@target
    return
        text {
            index-info:get-info-about("lit", $reference, "abbrev")
        }
};


declare function fontaneSimple:add-log-entry($log-file as xs:string,
$message as xs:string) as empty-sequence() {
  let $entry := <LogEntry timestamp="{util:system-time()}">{$message}</LogEntry>
  return update insert $entry into doc($log-file)/*
};
