xquery version "3.1";

(:~
 : This modules handles the conversion of the Fontante-TEI/XML into TEI simplePrint
 : for the edited text. The resulting TEI simplePrint is the basis for the "Editerter
 : Text" (edited text) view on the website and the book. It represents the latest
 : layer of text.
 :
 : @author Michelle Weidling
 : @version 0.1
 : @since TODO
 :)

module namespace tidySimple ="http://fontane-nb.dariah.eu/tidysimple";


declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace test="http://exist-db.org/xquery/xqsuite";

import module namespace config="http://textgrid.de/ns/SADE/config" at "../../config/config.xqm";
import module namespace functx="http://www.functx.com";
import module namespace index-info="http://fontane-nb.dariah.eu/index-info" at "index-info.xqm";
import module namespace presort="http://fontane-nb.dariah.eu/presort" at "presort.xqm";
import module namespace simpleHelpers="http://fontane-nb.dariah.eu/teisimplehelpers" at "teisimplehelpers.xqm";


declare variable $tidySimple:valid-hands :=
    for $res in collection($config:data-root || "/data")
    return
        $res//tei:handNote[@script = "contemporary"]/@xml:id/string();


declare function tidySimple:main($tei as node()*, $uri as xs:string) {
    let $tidy := tidySimple:enhance-handshifts($tei//tei:text)
        => tidySimple:sort-out-surplus-elements()
        => tidySimple:sort-out-invalid-hands()
        => tidySimple:split-headings()
        => tidySimple:summarize()
(:        => tidySimple:summarize-hi():)
    let $header :=
        tidySimple:get-Fontanes-sources($tei//tei:teiHeader[parent::tei:TEI])
        => tidySimple:get-references-in-abstract()
    let $id-parts := tokenize($tei//tei:TEI/@id, " ")
    let $key1 := substring($id-parts[2], 1, 1)
    let $key2 := substring($id-parts[2], 2)
    let $final-tei := <TEI xmlns="http://www.tei-c.org/ns/1.0" id="{$tei//tei:TEI/@id}" key1="{$key1}" key2="{$key2}">{$header}{$tidy}</TEI>
    let $store := xmldb:store($config:data-root || "/print/xml/", $uri || ".xml", $final-tei)
    return
        $final-tei
};

declare function tidySimple:sort-out-invalid-hands($nodes as node()*)
as node()* {
    for $node in $nodes return
        typeswitch ($node)
        case text() return
            if($node/preceding::tei:milestone[@unit = "handshift"][1]
            and not(simpleHelpers:is-hand-valid($tidySimple:valid-hands, $node/preceding::tei:milestone[@unit = "handshift"][1]))) then
                ()
            else
                $node

        (: all lines have to be preserved because of the editorial commentary
        which references the lines in the notebooks. if we omitted @unit = "line"
        referencing wouldn't work any longer :)
        case element(tei:milestone) return
            let $prev-handshift := $node/preceding::tei:milestone[@unit = "handshift"][1]
                return
                if($node/@unit = "handshift" and
                simpleHelpers:is-hand-valid($tidySimple:valid-hands, $node)) then
                    tidySimple:copy-element($node, "post")

                else if($node/@unit = "handshift") then
                    ()

                else if($prev-handshift
                and $node/@unit = "line"
                and not(simpleHelpers:is-hand-valid($tidySimple:valid-hands, $prev-handshift))) then
                    tidySimple:copy-element($node, "post")

                else if($prev-handshift
                and not(simpleHelpers:is-hand-valid($tidySimple:valid-hands, $prev-handshift))) then
                    ()

                else
                    tidySimple:copy-element($node, "post")

        case element(tei:div) return
            (: even though it's posthumous we want to keep the text written on
            calendar pages by Friedrich Fontane. Unfortunately, Friedrich's
            handshift is oftentimes not the first hand appearing on the page
            but we want to keep the page nevertheless. :)
            if($node/@type = "Kalenderblatt") then
                tidySimple:copy-element($node, "post")
            else
                tidySimple:invalid-hands-default-return($node)

        default return
            tidySimple:invalid-hands-default-return($node)
};

declare function tidySimple:invalid-hands-default-return($node as node()*)
as node()* {
    let $prev-handshift := $node/preceding::tei:milestone[@unit = "handshift"][1]
    let $first-child-handshift := $node/child::tei:milestone[@unit = "handshift"][1]
    let $first-child-element := $node/child::*[1]
    let $first-child-node := $node/child::node()[1]

    return
        (: in some cases the valid handshift is the first child node
        instead of a previous node. of course we want to keep the element
        then :)
        if($first-child-element = $first-child-handshift
        (: ensure there's no text before the handshift :)
        and (normalize-space($first-child-node) = ""
            or $first-child-element = $first-child-node)
        and simpleHelpers:is-hand-valid($tidySimple:valid-hands, $first-child-handshift)) then
            tidySimple:copy-element($node, "post")


        else if($prev-handshift
        and not(simpleHelpers:is-hand-valid($tidySimple:valid-hands, $prev-handshift))) then
            ()

        else
            tidySimple:copy-element($node, "post")
};


declare function tidySimple:sort-out-surplus-elements($nodes as node()*)
as node()* {
    for $node in $nodes return
        typeswitch ($node)
        case text() return
            $node

        case element(tei:milestone) return
            if($node/@unit = "handshift") then
                if(simpleHelpers:is-prev-hand-same($node)
                or not(tidySimple:has-hand-text($node))) then
                    ()
                else
                    element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                        $node/@*,
                        tidySimple:sort-out-surplus-elements($node/node())
                    }

            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:head) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:date) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:rs) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:note) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:abbr) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:list) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        case element(tei:item) return
            if(not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }


        case element(tei:div) return
            if($node/@type = "label"
            and not($node/* or $node/node())) then
                ()
            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:sort-out-surplus-elements($node/node())
                }

        default return
            element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                $node/@*,
                tidySimple:sort-out-surplus-elements($node/node())
            }
};

declare function tidySimple:has-hand-text($node as element(tei:milestone))
as xs:boolean {
    let $next-handshift := $node/following::tei:milestone[@unit = "handshift"][1]
    let $nodes-between := $node/following::node()[. << $next-handshift]
    let $is-text-node :=
        for $node-between in $nodes-between
        return
            if ($node-between[self::text()]
            and not(normalize-space($node-between) = "")) then
                true()
            else
                false()
    return
        if($next-handshift
        and functx:is-value-in-sequence(true(), $is-text-node)) then
            true()
        else if(not($next-handshift)) then
            true()
        else
            false()
};


declare function tidySimple:copy-element($node as node(), $flag as xs:string)
 {
    element {QName("http://www.tei-c.org/ns/1.0", $node/name())}{
        $node/@*,
        if($flag = "post") then
            tidySimple:sort-out-invalid-hands($node/node())
        else if($flag = "surplus") then
            tidySimple:sort-out-invalid-hands($node/node())
        else if($flag = "hs-enhance") then
            tidySimple:enhance-handshifts($node/node())
        else if($flag = "sources") then
            tidySimple:get-Fontanes-sources($node/node())
        else if($flag = "summarize") then
            tidySimple:summarize($node/node())
        else if($flag = "summarize-hi") then
            tidySimple:summarize-hi($node/node())
        else if($flag = "ref") then
            tidySimple:get-references-in-abstract($node/node())
        else
            text{"!!!Kopieren des Elements fehlgeschlagen!!!"}
    }
};

(:~
 : Purges surplus attributes from tei:milestone[@unit = "handshift"].
 :
 : @author Michelle Weidling
 : @param $node the current tei:milestone[@unit = "handshift"]
 : @return the purged tei:milestone[@unit = "handshift"]
 :  :)
declare function tidySimple:clear-handshift($node as element(tei:milestone))
as element(tei:milestone) {
    element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
        attribute unit {"handshift"},
        $node/(@* except (@subtype, @rend)),
        if($node/@subtype = "") then
            ()
        else
            $node/@subtype,
        if($node/@rend = "") then
            ()
        else
            $node/@rend
    }
};


declare function tidySimple:enhance-handshifts($nodes as node()*)
as node()* {
    for $node in $nodes return
        typeswitch ($node)
        case text() return
            $node

        case element(tei:milestone) return
            if($node/@unit = "handshift") then
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                        $node/(@* except (@script, @medium)),
                        let $script := $node/@script/string()
                        let $medium := $node/@medium/string()
                        let $enhanced-script := tidySimple:enhance-script($node, $script)
                        let $enhanced-medium := tidySimple:enhance-medium($node, $medium)
                        return
                            attribute rend {$enhanced-script || " " || $enhanced-medium}
                    }
            else
                tidySimple:copy-element($node, "hs-enhance")

        default return
             tidySimple:copy-element($node, "hs-enhance")
};

declare function tidySimple:enhance-script($node as node(), $script as xs:string?) as xs:string {
    if(not($script) or $script = "script()") then
        let $prev-relevant-hs := $node/preceding::tei:milestone[@unit = "handshift"][@script and not(@script = "script()")][1]
        return
            if($prev-relevant-hs) then
                $prev-relevant-hs/@script/string()
            else
                "Latf"
    else
        $script
};


declare function tidySimple:enhance-medium($node as node(), $medium as xs:string?) as xs:string {
    if(not($medium) or $medium = "medium()") then
        let $prev-relevant-hs := $node/preceding::tei:milestone[@unit = "handshift"][@medium and not(@medium = "medium()")][1]
        return
            if($prev-relevant-hs) then
                $prev-relevant-hs/@medium/string()
            else
                "pencil"
    else
        $medium
};



declare function tidySimple:split-headings($nodes as node()*) as node()* {
    for $node in $nodes return
        typeswitch($node)
        case text() return
            $node

        case element(tei:head) return
            if($node//tei:lb[@type = "edited_text"]) then
                let $lb := $node//tei:lb[@type = "edited_text"]
                let $split-top := $node/node()[. << $lb]
                let $split-bottom := $node/node()[. >> $lb]
                return
                    element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                        $node/@*,
                        (element tei:seg{
                            attribute type {"upper-part"},
                            $split-top
                        },
                        element tei:lb{
                            attribute type {"edited_text"}
                        },
                        element tei:seg{
                            attribute type {"lower-part"},
                            $split-bottom
                        })
                    }

            else
                element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                    $node/@*,
                    tidySimple:split-headings($node/node())
                }

        default return
            element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
                $node/@*,
                if($node/tei:add[@type = "edited_text" and @subtype = "interlinear"]) then
                    let $add := $node/tei:add[@type = "edited_text" and @subtype = "interlinear"]
                    return
                        (element tei:seg{
                            attribute type {"upper-part"},
                            $add/node()
                        },
                        element tei:lb{
                            attribute type {"edited_text"}
                        },
                        element tei:seg{
                            attribute type {"lower-part"},
                            $add/following-sibling::node()
                        })
                else
                    tidySimple:split-headings($node/node())
            }
};


declare function tidySimple:get-next-content($node as node()*, $content as node()*)
as node()* {
    if($node/@next) then
        let $next-hi-id := substring-after($node/@next, "#")
        let $next-node := $node/following-sibling::*[@xml:id = $next-hi-id]
        let $next-content := $next-node//text()[not(ancestor::tei:index)]
        let $break := if($node/@break = "true") then text{" "} else ()
        let $content := ($content, $break, $next-content)

        return
            tidySimple:get-next-content($next-node, $content)
    else
        $content
};

declare function tidySimple:get-Fontanes-sources($header as node()*) {
    for $node in $header return
        typeswitch ($node)

        case text() return
            $node

        case comment() return
            ()

        case element(tei:derivation) return
            element tei:derivation {
                for $ref in $node//tei:link
                    let $target := substring-after($ref/@target, " ")
                    let $abbrev := index-info:get-info-about("lit", $target, "abbrev")
                    let $confer-target := $ref/@resp/string()
                    let $confer :=
                        if($confer-target) then
                            index-info:get-info-about("lit", $confer-target, "abbrev")
                        else
                            ()
                    return
                        element tei:term {
                            $abbrev,
                            if($confer) then
                                "; vgl. " || $confer
                            else
                                ()
                        }
            }
        default return
            tidySimple:copy-element($node, "sources")
};



declare function tidySimple:summarize($nodes as node()*) as node()* {
    for $node in $nodes return
        typeswitch ($node)

        case text() return
            $node

        case comment() return
            $node

        case element(tei:milestone) return
            if($node/@unit = "line") then
                if($node/preceding::*[1][self::tei:milestone[@unit = "line"]]
                and $node/preceding::node()[1][self::tei:milestone[@unit = "line"]
                    or normalize-space(.) = ""]) then
                    ()
                else
                    tidySimple:copy-element($node, "summarize")
            else
                tidySimple:copy-element($node, "summarize")

        case element(tei:rs) return
            tidySimple:summarize-entries($node)

        case element(tei:list) return
            tidySimple:summarize-entries($node)

        case element(tei:item) return
            tidySimple:summarize-entries($node)

(:        case element(tei:hi) return:)
(:            tidySimple:summarize-entries($node):)

        case element(tei:seg) return
            tidySimple:summarize-entries($node)


        default return
            tidySimple:copy-element($node, "summarize")
};



declare function tidySimple:summarize-entries($node as node()) as node()* {
    (: the first element of a virtual aggregation:)
    if($node/@next and not($node/@prev)) then
        element {QName("http://www.tei-c.org/ns/1.0", $node/name())} {
            $node/(@* except @next),
            tidySimple:apply-all-nexts($node)
        }
    else if($node/@prev) then
        ()
    (: nodes that aren't part of the virtual aggregation :)
    else
        tidySimple:copy-element($node, "summarize")
};


declare function tidySimple:apply-all-nexts($node as node()) as node()* {
    (: entry point of virtual aggregation:)
    if($node/@next and not($node/@prev)) then
        let $next-node := tidySimple:find-corresp-node($node, "next")
        let $nodes-inbetween := $node/following::node()[. << $next-node]
        return
            if(count($next-node) = 1) then
                (tidySimple:summarize($node/node()),
                $nodes-inbetween,
                tidySimple:apply-all-nexts($next-node))
            else
                ()

    (: last of a virtual aggregation: exit point :)
    else if(not($node/@next)) then
        tidySimple:summarize($node/node())

    (: element in the middle of a virtual aggregation:)
    else
        let $next-node := tidySimple:find-corresp-node($node, "next")
        let $nodes-inbetween := $node/following::node()[. << $next-node]
        return
            if(count($next-node) = 1) then
                (tidySimple:summarize($node/node()),
                $nodes-inbetween,
                tidySimple:apply-all-nexts($next-node))
            else
                ()
};


declare function tidySimple:find-corresp-node($node as node(), $flag as xs:string) {
    let $target-id :=
        if($flag = "next") then
            tidySimple:get-id($node/@next)
        else if($flag = "prev") then
            tidySimple:get-id($node/@prev)
        else
            error(QName("FONTANE", "tidySimple001"), "Invalid flag: " || $flag || "." )
    return
        $node/root()//*[@xml:id = $target-id]
};


declare function tidySimple:get-id($target as xs:string) as xs:string {
    $target => replace("#", "")
};



(:
 : Resolves all tei:ptr in a tei:abstract (which is displayed in the Ueberblicks-
 : kommentar) and places the resp. abbreviated bibliographical reference instead.
 :
 : @param $nodes Nodes within the tei:teiHeader
 : @return processed nodes
 : @author Michelle Weidling
 :)
declare function tidySimple:get-references-in-abstract($nodes as node()*) as node()* {
    for $node in $nodes return
        typeswitch ($node)

        case text() return
            $node

        case comment() return
            ()

        case element(tei:ptr) return
            if($node/ancestor::tei:abstract) then
                let $target-id := replace($node/@target, "#", "")
                let $abbrev := index-info:get-info-about("lit", $target-id, "abbrev")
                return
                    text{$abbrev || " "}
            else
                tidySimple:copy-element($node, "ref")

        default return
            tidySimple:copy-element($node, "ref")
};

declare function tidySimple:summarize-hi($nodes as node()*) as node()* {
    for $node in $nodes return
        typeswitch ($node)

        case text() return
            $node

        case comment() return
            $node

        case element(tei:hi) return
            tidySimple:summarize-entries($node)

        default return
            tidySimple:copy-element($node, "summarize-hi")
};

declare function tidySimple:summarize-hi-entries($node as node()) as node()* {
    (: the first element of a virtual aggregation:)
    if($node/@next and not($node/@prev)) then
        element tei:hi {
            $node/(@* except @next),
            tidySimple:apply-all-hi-nexts($node)
        }
    else if($node/@prev) then
        ()
    (: nodes that aren't part of the virtual aggregation :)
    else
        tidySimple:copy-element($node, "summarize-hi")
};


declare function tidySimple:apply-all-hi-nexts($node as node()) as node()* {
    (: entry point of virtual aggregation:)
    if($node/@next and not($node/@prev)) then
        let $next-node := tidySimple:find-corresp-node($node, "next")
        let $nodes-inbetween := $node/following::node()[. << $next-node]
        return
            if(count($next-node) = 1) then
                (tidySimple:summarize-hi($node/node()),
                $nodes-inbetween,
                tidySimple:apply-all-hi-nexts($next-node))
            else
                ()

    (: last of a virtual aggregation: exit point :)
    else if(not($node/@next)) then
        tidySimple:summarize-hi($node/node())

    (: element in the middle of a virtual aggregation:)
    else
        let $next-node := tidySimple:find-corresp-node($node, "next")
        let $nodes-inbetween := $node/following::node()[. << $next-node]
        return
            if(count($next-node) = 1) then
                (tidySimple:summarize-hi($node/node()),
                $nodes-inbetween,
                tidySimple:apply-all-hi-nexts($next-node))
            else
                ()
};
